#
# This file is part of Efforia Open Source Initiative.
#
# Copyright (C) 2011-2014 William Oliveira de Lagos <william@efforia.com.br>
#
# Shipping is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Shipping is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Shipping. If not, see <http://www.gnu.org/licenses/>.
#

from django.db.models import *
from django.conf import settings
from django.contrib.auth.models import User
from django.template import Context,Template
from datetime import date

locale = settings.LOCALE_DATE

class DeliverableProperty(Model):
    sku = CharField(default='',max_length=20)
    height = IntegerField(default=16)
    length = IntegerField(default=16)
    width = IntegerField(default=16)
    weight = FloatField(default=0.1)

class Deliverable(Model):
    name = CharField(default='((',max_length=50)
    user = ForeignKey(User,related_name='+')
    product = IntegerField(default=1)
    mail_code = CharField(default='',max_length=100)
    height = IntegerField(default=1)
    length = IntegerField(default=1)
    width = IntegerField(default=1)
    weight = IntegerField(default=10)
    value = FloatField(default=0.0)
    date = DateTimeField(default=date.today(),auto_now_add=True)
    def token(self): return self.name[:2]
    def name_trimmed(self): return self.name.split(';')[0][1:]
    def month(self): return locale[self.date.month-1]


#
# This file is part of Efforia Open Source Initiative.
#
# Copyright (C) 2011-2014 William Oliveira de Lagos <william@efforia.com.br>
#
# Shipping is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Shipping is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Shipping. If not, see <http://www.gnu.org/licenses/>.
#
#!/usr/bin/python
# -*- coding: utf-8 -*-
# Write your views here.

from shipping.email import send_invoice
from django.http import HttpResponse as response

def main(request):
	return response('Hello World!')

def mail_send(request):
	return send_invoice(request)


#
# This file is part of Efforia Open Source Initiative.
#
# Copyright (C) 2011-2014 William Oliveira de Lagos <william@efforia.com.br>
# Copyright (C) 2010 Robson Silva - r@linux.com
#
# Shipping is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Shipping is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Shipping. If not, see <http://www.gnu.org/licenses/>.
#
#!/usr/bin/python
# -*- coding: utf-8 -*-


import urllib
from beautifulsoup import BeautifulStoneSoup as BSS

FORMATOS = {
    'PACOTE': 1,
    'ROLO': 2,
}

SERVICOS = {
    'PAC':'41106',
    'SEDEX':'40010',
    'SEDEX10':'40215',
    'SEDEXHOJE':'40290',
    'SEDEXCOBRAR':'40045',
    'ALL': '41106,40010,40215,40290,40045',
}

class CorreiosShippingService(object):
    def __init__(self, cep_origem='80050370'):
        #ceps
        self.cep_origem = cep_origem
        self.cep_destino = None
        
        #servicos
        self.aviso_recebimento = 'N'
        self.valor_declarado = 0
        self.mao_propria = 'N'
        self.servico = 'ALL'

        #medidas
        self.formato = 'PACOTE'
        self.altura = 0
        self.largura = 0
        self.comprimento = 0
        self.diametro = 0
        self.peso = 0.3

        #configs
        self.empresa = '' #id da empresa, quando possui contrato
        self.senha = ''   #senha
        self.tipo = 'xml'
        self.URL = 'http://ws.correios.com.br/calculador/CalcPrecoPrazo.aspx'
        
        self.response = None
        self.hash = {}
        self.results = {}
        self.errors = {}

    def __call__(self, cep_destino=None, servico=None,):
        """
        Call :)
        """
        if cep_destino:
            self.cep_destino = cep_destino

        # pega o codigo do servico p/ envio ao webservice
        if servico and SERVICOS.has_key(servico):
            self.servico = servico
        
        # valida medidas e pesos minimos
        self._validate()

        # prepara o hash para envio
        self._build_hash()

        # envia os parametros e recupera o retorno
        self._request()

        # processa o resultado
        self._parse()


    def _build_hash(self):
        """
        Cria hash com dados instanciados
        """
        h = self.hash
        h['nCdEmpresa'] = self.empresa
        h['sDsSenha'] = self.senha
        h['strRetorno'] = self.tipo
        h['sCdMaoPropria'] = self.mao_propria
        h['nVlValorDeclarado'] = self.valor_declarado
        h['sCdAvisoRecebimento'] = self.aviso_recebimento
        h['nCdFormato'] = FORMATOS[self.formato]
        h['sCepOrigem'] = self.cep_origem
        h['sCepDestino'] = self.cep_destino
        h['nCdServico'] = SERVICOS[self.servico]
        h['nVlAltura'] = self.altura
        h['nVlLargura'] = self.largura
        h['nVlComprimento'] = self.comprimento
        h['nVlDiametro'] = self.diametro
        h['nVlPeso'] = self.peso
        self.hash = h


    def _validate(self):
        """ Valida as medidas (apenas os minimos)
        para calculo
        """

        peso_minimo = 0.3
        if self.formato == 'ROLO':
            comprimento_minimo = 18
            diametro_minimo = 5
            altura_minima = 0
            largura_minima = 0
        else:
            comprimento_minimo = 16
            diametro_minimo = 0
            altura_minima = 2
            largura_minima = 5

        if self.diametro < diametro_minimo:
            self.diametro = diametro_minimo

        if self.altura < altura_minima:
            self.altura = altura_minima

        if self.comprimento < comprimento_minimo:
            self.comprimento = comprimento_minimo

        # altura nao pode ser maior que comprimento
        if self.altura > self.comprimento:
            self.altura = self.comprimento

        if self.largura < largura_minima:
            self.largura = largura_minima

        # largura nao pode ser menor que 11cm quando o comprimento
        # for menor que 25cm (apenas no formato PACOTE)
        if self.formato == 'PACOTE' and self.largura < 11 \
            and self.comprimento < 25:
            self.largura = 11

        if self.peso < peso_minimo:
            self.peso = peso_minimo


    def _request(self):
        """ 
        Realiza a requisicao ao webservice e recupera
        o retorno
        """
        url = '%s?%s' %(self.URL, urllib.urlencode(self.hash))
        self.response = urllib.urlopen(url).read()


    def _parse(self):
        """
        Processa o xml retornado pelo webservice
        utilizando o BeautifulSoup

        Exemplo do retorno:
            <cServico>
                <Codigo>40045</Codigo>
                <Valor>12,10</Valor>
                <PrazoEntrega>1</PrazoEntrega>
                <ValorMaoPropria>0,00</ValorMaoPropria>
                <ValorAvisoRecebimento>0,00</ValorAvisoRecebimento>
                <ValorValorDeclarado>1,00</ValorValorDeclarado>
                <EntregaDomiciliar>S</EntregaDomiciliar>
                <EntregaSabado>S</EntregaSabado>
                <Erro>0</Erro>
                <MsgErro></MsgErro>
            </cServico>
        """
        self.xmltree = BSS(self.response, selfClosingTags=[],
                    convertEntities=BSS.ALL_ENTITIES)

        for result in self.xmltree('cservico'):
            servico_id = result('codigo')[0].contents[0]
            prazo = result('prazoentrega')[0].contents[0]
            valor = result('valor')[0].contents[0]
            erro = result('erro')[0].contents[0]
            servico = [k for k in SERVICOS if SERVICOS[k] == servico_id][0]

            #outras opcoes disponiveis no retorno:
            #valor_mao_propria = result('valormaopropria')[0].contents[0]
            #valor_aviso_recebimento = result('valoravisorecebimento')[0].contents[0]
            #valor_valor_declarado = result('valorvalordeclarado')[0].contents[0]
                        
            if erro != u'0':
                msgerro = result('msgerro')[0].contents[0]
                self.errors[servico] = msgerro
            else:
                self.results[servico] = prazo, valor


    def print_results(self):
        """
        Imprime o resultado no terminal
        """
        if self.results:
            print 'Resultados:'
            for k, v in self.results.iteritems():
                prazo, valor = v
                print '%s - %s dias - R$ %s' %(k, prazo, valor)
        
        if self.errors:
            print 'Erros:'
            for k, v in self.errors.iteritems():
                print '%s - %s' %(k, v)



#
# This file is part of Efforia Open Source Initiative.
#
# Copyright (C) 2011-2014 William Oliveira de Lagos <william@efforia.com.br>
#
# Shipping is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Shipping is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Shipping. If not, see <http://www.gnu.org/licenses/>.
#
#!/usr/bin/python
# -*- coding: utf-8 -*-

from urllib2 import Request,urlopen
from xml.dom import minidom as dom

def create_deliverable(sender,receiver,width,height,length,weight):
	if int(height) < 2: return 'Altura abaixo do mínimo (2cm).'
	if int(width) < 11: return 'Largura abaixo do mínimo (11cm).'
	if int(length) < 16: return 'Profundidade abaixo do mínimo (16cm).'
	deliverable = {
		'sender': sender,
		'receiver': receiver,
		'width': str(width),
		'height': str(height),
		'length': str(length),
		'weight': str(weight),
	}	
	return deliverable

def build_request(d):
	url = 'https://ff.paypal-brasil.com.br/FretesPayPalWS/WSFretesPayPal'
	headers = {
		'Content-Type': 'text/xml; charset=utf-8',
		'SoapAction': '%s/getPreco' % url
	}
	xml = """<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:shipping=\"https://ff.paypal-brasil.com.br/FretesPayPalWS\">
        <soapenv:Header />
          <soapenv:Body>
            <shipping:getPreco>
              <cepOrigem>%s</cepOrigem>
              <cepDestino>%s</cepDestino>
              <largura>%s</largura>
              <altura>%s</altura>
              <profundidade>%s</profundidade>
              <peso>%s</peso>
            </shipping:getPreco>
          </soapenv:Body>
        </soapenv:Envelope>""" % (d['sender'],d['receiver'],d['width'],d['height'],d['length'],d['weight'])
	return url,headers,xml

def delivery_value(deliverable,tornado=False):
	url,headers,xml = build_request(deliverable)
	if tornado:
		import tornadoweb
		value = tornadoweb.request(url,headers,xml)
	else:
		req = Request(url,xml,headers)
		value = urlopen(req).read()
	val = dom.parseString(value).getElementsByTagName('return')[0].childNodes[0].wholeText
	if '-2.0' in val: return 'Não foi possível calcular o frete.'
	else: return '%.2f' % float(val)



#
# This file is part of Efforia Open Source Initiative.
#
# Copyright (C) 2011-2014 William Oliveira de Lagos <william@efforia.com.br>
#
# Shipping is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Shipping is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Shipping. If not, see <http://www.gnu.org/licenses/>.
#
#!/usr/bin/python
# -*- coding: utf-8 -*-
from beautifulsoup import BeautifulSoup
import cookielib
import re
import urllib
import urllib2

URL_CORREIOS = 'http://www.buscacep.correios.com.br/servicos/dnec/'

class CorreiosCode():
    def __init__(self, proxy=None):
        cj = cookielib.LWPCookieJar()
        cookie_handler = urllib2.HTTPCookieProcessor(cj)
        if proxy:
            proxy_handler = urllib2.ProxyHandler({'http': proxy})
            opener = urllib2.build_opener(proxy_handler, cookie_handler)
        else:
            opener = urllib2.build_opener(cookie_handler)
        urllib2.install_opener(opener)

    def _url_open(self, url, data=None, headers=None):
        if headers == None:
            headers = {}

        headers['User-agent'] = 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)'
        encoded_data = urllib.urlencode(data) if data else None
        url = URL_CORREIOS + url

        req = urllib2.Request(url, encoded_data, headers)
        handle = urllib2.urlopen(req)

        return handle

    def _parse_detalhe(self, html):
        soup = BeautifulSoup(html.decode('ISO-8859-1'))

        value_cells = soup.findAll('td', attrs={'class': 'value'})
        values = [cell.firstText(text=True) for cell in value_cells]
        localidade, uf = values[2].split('/')
        values_dict = {
            'Logradouro': values[0],
            'Bairro': values[1],
            'Localidade': localidade,
            'UF': uf,
            'CEP': values[3]
        }
        return values_dict

    def _parse_linha_tabela(self, tr):
        values = [cell.firstText(text=True) for cell in tr.findAll('td')]
        keys = ['Logradouro', 'Bairro', 'Localidade', 'UF', 'CEP']
        return dict(zip(keys, values))

    def _parse_tabela(self, html):
        soup = BeautifulSoup(html)
        linhas = soup.findAll('tr', attrs={
            'onclick': re.compile(r"javascript:detalharCep\('\d+','\d+'\);")
        })
        return [self._parse_linha_tabela(linha) for linha in linhas]

    def _parse_faixa(self, html):
        if u"não está cadastrada" in html.decode('cp1252'):
            return None
        ceps = re.findall('\d{5}-\d{3}', html)
        if len(ceps) == 4 or len(ceps) == 6: #uf (+ uf) + cidade com range
            return tuple(ceps[-2:])
        elif len(ceps) == 3 or len(ceps) == 5: #uf (+ uf) + cidade com cep único
            return ceps[-1]
        else:
            raise ValueError("HTML recebido não é válido")

    def detalhe(self, posicao=0):
        """Retorna o detalhe de um CEP da última lista de resultados"""
        handle = self._url_open('detalheCEPAction.do', {'Metodo': 'detalhe',
                                                        'TipoCep': 2,
                                                        'Posicao': posicao + 1,
                                                        'CEP': ''})
        html = handle.read()
        return self._parse_detalhe(html)

    def consulta_faixa(self, localidade, uf):
        """Consulta site e retorna faixa para localidade"""
        url = 'consultaFaixaCepAction.do'
        data = {
            'UF': uf,
            'Localidade': localidade.encode('cp1252'),
            'cfm': '1',
            'Metodo': 'listaFaixaCEP',
            'TipoConsulta': 'faixaCep',
            'StartRow': '1',
            'EndRow': '10',
        }
        html = self._url_open(url, data).read()
        return self._parse_faixa(html)

    def consulta(self, endereco, primeiro=False,
                 uf=None, localidade=None, tipo=None, numero=None):
        """Consulta site e retorna lista de resultados"""

        if uf is None:
            url = 'consultaEnderecoAction.do'
            data = {
                'relaxation': endereco.encode('ISO-8859-1'),
                'TipoCep': 'ALL',
                'semelhante': 'N',
                'cfm': 1,
                'Metodo': 'listaLogradouro',
                'TipoConsulta': 'relaxation',
                'StartRow': '1',
                'EndRow': '10'
            }
        else:
            url = 'consultaLogradouroAction.do'
            data = {
                'Logradouro': endereco.encode('ISO-8859-1'),
                'UF': uf,
                'TIPO': tipo,
                'Localidade': localidade.encode('ISO-8859-1'),
                'Numero': numero,
                'cfm': 1,
                'Metodo': 'listaLogradouro',
                'TipoConsulta': 'logradouro',
                'StartRow': '1',
                'EndRow': '10'
            }

        h = self._url_open(url, data)
        html = h.read()

        if primeiro:
            return self.detalhe()
        else:
            return self._parse_tabela(html)



#
# This file is part of Efforia Open Source Initiative.
#
# Copyright (C) 2011-2014 William Oliveira de Lagos <william@efforia.com.br>
#
# Shipping is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Shipping is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Shipping. If not, see <http://www.gnu.org/licenses/>.
#
#!/usr/bin/python
# -*- coding: utf-8 -*-

import re,urllib2
from beautifulsoup import BeautifulSoup

class EncomendaRepository(object):
        
    def __init__(self):
        from scraping import CorreiosWebsiteScraper
        self.correios_website_scraper = CorreiosWebsiteScraper()
    
    def get(self, numero):
        return self.correios_website_scraper.get_encomenda_info(numero)

class Encomenda(object):
    
    def __init__(self, numero):
        self.numero = numero
        self.status = []
    
    def adicionar_status(self, status):
        self.status.append(status)
        self.status.sort(lambda x, y: 1 if x.data > y.data else -1)
    
    def ultimo_status_disponivel(self):
        return self.status[len(self.status) - 1] if len(self.status) > 0 else None

    def primeiro_status_disponivel(self):
        return self.status[0] if len(self.status) > 0 else None

class Status(object):
    
    def __init__(self, **kwargs):
        self.data = kwargs.get('data')
        self.local = kwargs.get('local')
        self.situacao = kwargs.get('situacao')
        self.detalhes = kwargs.get('detalhes')

class Correios:
    def __init__(self): pass

class CorreiosWebsiteScraper(object):
    
    def __init__(self, http_client=urllib2):
        self.url = 'http://websro.correios.com.br/sro_bin/txect01$.QueryList?P_ITEMCODE=&P_LINGUA=001&P_TESTE=&P_TIPO=001&P_COD_UNI='
        self.http_client = http_client
        
    def get_encomenda_info(self, numero):
        request = self.http_client.urlopen('%s%s' % (self.url, numero))
        html = request.read()
        request.close()
        if html:
            encomenda = Encomenda(numero)
            [encomenda.adicionar_status(status) for status in self._get_all_status_from_html(html)]
            return encomenda
    
    def _get_all_status_from_html(self, html):
        html_info = re.search('.*(<table.*</TABLE>).*', html, re.S)
	try:
	        table = html_info.group(1)
	except AttributeError,e:
		return [-1]
        
        soup = BeautifulSoup(table)
        
        status = []
        count = 0
        for tr in soup.table:
            if count > 4 and str(tr).strip() != '':
                if re.match(r'\d{2}\/\d{2}\/\d{4} \d{2}:\d{2}', tr.contents[0].string):
                    status.append(
                            Status(data=unicode(tr.contents[0].string),
                                    local=unicode(tr.contents[1].string),
                                    situacao=unicode(tr.contents[2].font.string))
                    )
                else:
                    status[len(status) - 1].detalhes = unicode(tr.contents[0].string)
                    
            count = count + 1
        
        return status



#
# This file is part of Efforia Open Source Initiative.
#
# Copyright (C) 2011-2014 William Oliveira de Lagos <william@efforia.com.br>
#
# Shipping is free software: you can redistribute it and/or modify
# it under the terms of the Lesser GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Shipping is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Shipping. If not, see <http://www.gnu.org/licenses/>.
#
#!/usr/bin/python
# -*- coding: utf-8 -*-

from django.utils.translation import ugettext as _

try:
    from mezzanine.conf import settings
    from cartridge.shop.utils import set_shipping
    from cartridge.shop.models import Cart
except ImportError,e:
    pass

from shipping.fretefacil import FreteFacilShippingService
from shipping.models import DeliverableProperty

def fretefacil_shipping_handler(request, form, order=None):
    if request.session.get("free_shipping"): return
    settings.use_editable()
    try: user_postcode = form.shipping_detail_postcode
    except AttributeError,e: user_postcode = settings.CLIENT_POSTCODE
    shippingservice = FreteFacilShippingService()
    cart = Cart.objects.from_request(request)
    delivery_value = 0.0
    if cart.has_items():
        for product in cart:
            properties = DeliverableProperty.objects.filter(sku=product.sku)
            if len(properties) > 0:
                props = properties[0]
                deliverable = shippingservice.create_deliverable(settings.STORE_POSTCODE,
                                                                 user_postcode,
                                                                 props.width,
                                                                 props.height,
                                                                 props.length,
                                                                 props.weight)
                delivery_value += float(shippingservice.delivery_value(deliverable))
    set_shipping(request, _("Correios"),delivery_value)